// Acquire analog potentiometers values to set the digital potentiometer value
// based on the filter settings.
// This is a test sketch for Arduino UNO and Duemilanove to test and setup the digital potentiometer
// Arduino Shield.
// Created on March 2015 by balearicdynamics@gmail.com http://www.balearicdynamics.com

// Include SPI Library
#include <SPI.h>
// Include the Streaming library for better serial output
#include <Streaming.h>

#define MAXPOT 6  // Max number of potentiometers
#define SERIALMON  // Undef SERIALMON to avoid serial output. For testing purposes only

// Units to show the potentiometer resistance value.
#define KOHM  0  // Set to 0 if the resistance should be in Ohm instead of K
// Units to show the digital potentiometer resisance value.
#define DKOHM 1 // Set to 0 if the resistance should be in Ohm instead of K

// Test analog potentiometers values in Ohm
#define ANALOGPOT  1000.00

// Digital potentiometer value in Ohm
#define DIGITALPOT  100000.00

// Steps divider from analog converter
#define ANALOGDIVIDER 1024.00

// Steps divider of the digital potentiometer
#define DIGITALDIVIDER 256

// SPI Clock frequency. Higher is the clock frequency, higher is the digital potentiometer
// response. Value is expressed in reading-per-second
// The frequency calculation is an integer rounded.
#define DIGIPOT_FREQ  8

// Set the analog pins array conected to the potentiometers
// Changing the order of the pins in the array changes the relationship between
// the analog read value and the digital generated output.
const int AnalogPotPort[] = { 0, 1, 2, 3, 4, 5 };

// set the slave select for the digital pot
const int slaveSelectPin = 10;

// Initial potentiometer value
int pValue = 0;

// Your setup initialization here
void setup()
{

  // set the output pins
  pinMode (slaveSelectPin, OUTPUT);

  // initialize SPI:
  SPI.begin();

#ifdef SERIALMON
  // Initialize the serial (for testing purposes)
  Serial.begin(9600);
#endif

}

// Main application loop
// This sample focus the attention on the digital potentiometer management.
// Several improvements can be added, e.g. the use of one interrupt instead
// a delay() call to cycle the SPI frequency.
void loop(void)
{

  // Potentiomenter read-write on all the values
  for (int i = 0; i < MAXPOT; i++) {

    // Get the input potentiometer analog value
    pValue = analogRead(AnalogPotPort[i]);
    // Now we convert the 10 bits value to the corresponding potentiometer value
    // as the opposite of the current value read
    double analogPot = ANALOGPOT - (ANALOGPOT / ANALOGDIVIDER * pValue);

#ifdef SERIALMON
    // Monitor the read value
    if (KOHM == 0) {
      // Value in Ohm
      Serial << "Pot[" << i << "] = " << analogPot << " - " << pValue << " || ";
    }
    else {
      // Value in K
      Serial << "Pot[" << i << "] = " << analogPot / 1000 << "K || ";
    }
#endif

    // Convert the value to the corresponding digital value.
    // Depending on the digital potentiometer we are using, the 256
    // steps have different increment value.
    // As we read a variation of max 1024 steps in the analog port we should
    // reduce the resolution to 256th of step.
    int dValue = map(pValue, 0, ANALOGDIVIDER, 0, DIGITALDIVIDER);

    // Now we convert the mapped value to the corresponding digital potentiometer
    // resistance. As we have mapped the read current value, the resistance is the
    // opposite as the read value.
    // For further development: the value can be tested empirycally to find a more precise
    // repsonse curve and define a progressive correciton value.
    double digitalPot = DIGITALPOT - (DIGITALPOT / DIGITALDIVIDER * dValue);

#ifdef SERIALMON
    // Monitor the read value
    if (DKOHM == 0) {
      // Value in Ohm
      Serial << "Digi[" << i << "] = " << digitalPot << " - " << dValue << " || ";
    }
    else {
      Serial << "Digi[" << i << "] = " << digitalPot / 1000 << "K || ";
    }
    // Value in K
#endif


    // Write the read value to the digital potentiometer
    digitalPotWrite(i, pValue);
  } // End of potentiometers read/write loop

#ifdef SERIALMON
  Serial.println("");
#endif

  // Wait for the next cycle
  delay(1000 / DIGIPOT_FREQ);

}


// Write the resistance value to the selected digital potentiometer
// The resistance value is converted to the range accepted by the
// digital potentiometer.
void digitalPotWrite(int address, int value) {
  // take the SS pin low to select the chip:
  digitalWrite(slaveSelectPin, LOW);
  //  send in the address and value via SPI:
  SPI.transfer(address);
  SPI.transfer(value);
  // take the SS pin high to de-select the chip:
  digitalWrite(slaveSelectPin, HIGH);
}
